@@ -149,6 +149,38 @@ impl File {
     }
 }
 
+fn find_interpolate(s: &str) -> Result<Option<(usize, bool)>> {
+    match s.find("##") {
+        Some(idx) => {
+            if s.find('#').unwrap() != idx || s[idx + 2..].find('#').is_some() {
+                return Err(anyhow!("Multiple interpolates in '{}' not supported", s));
+            }
+            Ok(Some((idx, true)))
+        }
+        None => match s.find('#') {
+            Some(idx) => {
+                if s[idx + 1..].find('#').is_some() {
+                    return Err(anyhow!("Multiple interpolates in '{}' not supported", s));
+                }
+                Ok(Some((idx, false)))
+            }
+            None => Ok(None),
+        },
+    }
+}
+
+fn replace_interpolate(s: &str, replacement: &str) -> String {
+    if let Some((_, double)) = find_interpolate(s).unwrap() {
+        if double {
+            s.replace("##", replacement)
+        } else {
+            s.replace('#', replacement)
+        }
+    } else {
+        String::from(s)
+    }
+}
+
 pub struct Runner<'a> {
     // ui: &'a ChompUI,
     cwd: String,
@@ -186,21 +218,18 @@ impl<'a> Job {
             if task.targets.len() > 0 {
                 match task.targets.iter().find(|&t| t.contains('#')) {
                     Some(interpolate_target) => {
-                        interpolate_target.replace('#', &self.interpolate.as_ref().unwrap())
+                        replace_interpolate(interpolate_target, &self.interpolate.as_ref().unwrap())
                     }
-                    None => task
-                        .deps
-                        .iter()
-                        .find(|&d| d.contains('#'))
-                        .unwrap()
-                        .replace('#', &self.interpolate.as_ref().unwrap()),
+                    None => replace_interpolate(
+                        task.deps.iter().find(|&d| d.contains('#')).unwrap(),
+                        &self.interpolate.as_ref().unwrap(),
+                    ),
                 }
             } else {
-                task.deps
-                    .iter()
-                    .find(|&d| d.contains('#'))
-                    .unwrap()
-                    .replace('#', &self.interpolate.as_ref().unwrap())
+                replace_interpolate(
+                    task.deps.iter().find(|&d| d.contains('#')).unwrap(),
+                    &self.interpolate.as_ref().unwrap(),
+                )
             }
         } else if self.targets.len() > 0 {
             self.targets.first().unwrap().to_string()
@@ -523,7 +552,7 @@ impl<'a> Runner<'a> {
                 let name = if is_interpolate_target && name.contains('#') {
                     // interpolates support "#" in the name as well
                     // which is treated as blank for the all case
-                    name.replace('#', "")
+                    replace_interpolate(name, "")
                 } else {
                     name.to_string()
                 };
@@ -532,7 +561,7 @@ impl<'a> Runner<'a> {
                 }
             } else if name.contains('#') {
                 // interpolate individual names only expanded when using "#" in the name
-                let name = name.replace('#', interpolate.as_ref().unwrap());
+                let name = replace_interpolate(name, interpolate.as_ref().unwrap());
                 if !self.task_jobs.contains_key(&name) {
                     self.task_jobs.insert(name, num);
                 }
@@ -555,7 +584,7 @@ impl<'a> Runner<'a> {
                         if !target.contains('#') {
                             continue;
                         }
-                        target.replace('#', interpolate)
+                        replace_interpolate(target, interpolate)
                     }
                     None => target.to_string(),
                 };
@@ -946,7 +975,7 @@ impl<'a> Runner<'a> {
         let target = if task.targets.len() == 0 {
             "".to_string()
         } else if let Some(interpolate) = &job.interpolate {
-            task.targets[target_index].replace('#', interpolate)
+            replace_interpolate(&task.targets[target_index], interpolate)
         } else {
             task.targets[target_index].clone()
         };
@@ -970,7 +999,10 @@ impl<'a> Runner<'a> {
                 .find(|(_, d)| d.contains('#'))
                 .unwrap()
                 .0;
-            vec![task.deps[interpolate_index].replace('#', interpolate)]
+            vec![replace_interpolate(
+                &task.deps[interpolate_index],
+                interpolate,
+            )]
         } else {
             vec![]
         };
@@ -1365,13 +1397,16 @@ impl<'a> Runner<'a> {
                                 node_num,
                                 mtime,
                                 Some(cmd_time),
-                                matches!(
+                                matches!(validation, ValidationCheck::NotOk) || matches!(
                                     validation,
                                     ValidationCheck::TargetsOnly | ValidationCheck::OkTargets
                                 ) && mtime.is_none(),
                             );
                         }
                         ExecState::Failed => match validation {
+                            ValidationCheck::NotOk => {
+                                self.mark_complete(node_num, mtime, Some(cmd_time), false)
+                            }
                             ValidationCheck::OkOnly | ValidationCheck::OkTargets => {
                                 self.mark_complete(node_num, mtime, Some(cmd_time), true)
                             }
@@ -1421,7 +1456,7 @@ impl<'a> Runner<'a> {
                 }
                 Ok(())
             }
-            _ => panic!("Unexpected promise transition state")
+            _ => panic!("Unexpected promise transition state"),
         }
     }
 
@@ -1456,9 +1491,9 @@ impl<'a> Runner<'a> {
                     let job = self.get_job(*job_num).unwrap();
                     let job_task = &self.tasks[job.task];
                     if let Some(name) = &job_task.name {
-                        if let Some(interpolate_idx) = name.find('#') {
+                        if let Some((interpolate_idx, double)) = find_interpolate(name)? {
                             let lhs = &name[0..interpolate_idx];
-                            let rhs = &name[interpolate_idx + 1..];
+                            let rhs = &name[interpolate_idx + if double { 2 } else { 1 }..];
                             if task.starts_with(lhs)
                                 && task.len() > lhs.len() + rhs.len()
                                 && task.ends_with(rhs)
@@ -1480,11 +1515,10 @@ impl<'a> Runner<'a> {
                 match interpolate_match {
                     Some((job_num, interpolate)) => {
                         let task_deps = &self.tasks[self.get_job(job_num).unwrap().task].deps;
-                        let input = task_deps
-                            .iter()
-                            .find(|dep| dep.contains("#"))
-                            .unwrap()
-                            .replace("#", interpolate);
+                        let input = replace_interpolate(
+                            task_deps.iter().find(|dep| dep.contains("#")).unwrap(),
+                            interpolate,
+                        );
                         let num = self
                             .expand_interpolate_match(
                                 watcher,
@@ -1534,9 +1568,9 @@ impl<'a> Runner<'a> {
                 let mut interpolate_rhs_match_len = 0;
                 for job_num in &self.interpolate_nodes {
                     if let Some(interpolate) = self.get_interpolate_target(*job_num) {
-                        let interpolate_idx = interpolate.find('#').unwrap();
+                        let (interpolate_idx, double) = find_interpolate(interpolate)?.unwrap();
                         let lhs = &interpolate[0..interpolate_idx];
-                        let rhs = &interpolate[interpolate_idx + 1..];
+                        let rhs = &interpolate[interpolate_idx + if double { 2 } else { 1 }..];
                         if target.starts_with(lhs)
                             && target.len() > lhs.len() + rhs.len()
                             && target.ends_with(rhs)
@@ -1557,11 +1591,10 @@ impl<'a> Runner<'a> {
                 match interpolate_match {
                     Some((job_num, interpolate)) => {
                         let task_deps = &self.tasks[self.get_job(job_num).unwrap().task].deps;
-                        let input = task_deps
-                            .iter()
-                            .find(|dep| dep.contains("#"))
-                            .unwrap()
-                            .replace("#", interpolate);
+                        let input = replace_interpolate(
+                            task_deps.iter().find(|dep| dep.contains("#")).unwrap(),
+                            interpolate,
+                        );
                         let num = self
                             .expand_interpolate_match(
                                 watcher,
@@ -1602,7 +1635,12 @@ impl<'a> Runner<'a> {
         assert!(has_glob_chars(target));
         let task_pattern = target.as_bytes()[0] as char == ':';
         let target = if task_pattern { &target[1..] } else { target };
-        let target_pattern = Pattern::new(target).unwrap();
+        let target_pattern = match Pattern::new(target) {
+            Ok(pattern) => pattern,
+            Err(e) => {
+                return Err(anyhow!("Unable to parse pattern {}, {}", target, e.msg));
+            }
+        };
 
         // Determine non-glob prefix and suffix of the target
         let mut target_prefix_len = 0;
@@ -1630,9 +1668,9 @@ impl<'a> Runner<'a> {
                 let task_num = job.task;
                 let job_task = &self.tasks[task_num];
                 if let Some(name) = &job_task.name {
-                    if let Some(interpolate_idx) = name.find('#') {
+                    if let Some((interpolate_idx, double)) = find_interpolate(name)? {
                         let lhs = &name[0..interpolate_idx];
-                        let rhs = &name[interpolate_idx + 1..];
+                        let rhs = &name[interpolate_idx + if double { 2 } else { 1 }..];
 
                         let maybe_intersects = if lhs.len() > target_prefix.len() {
                             lhs.starts_with(target_prefix)
@@ -1679,9 +1717,9 @@ impl<'a> Runner<'a> {
             let mut expansions = Vec::new();
             for job_num in &self.interpolate_nodes {
                 if let Some(interpolate) = self.get_interpolate_target(*job_num) {
-                    let interpolate_idx = interpolate.find('#').unwrap();
+                    let (interpolate_idx, double) = find_interpolate(interpolate).unwrap().unwrap();
                     let lhs = &interpolate[0..interpolate_idx];
-                    let rhs = &interpolate[interpolate_idx + 1..];
+                    let rhs = &interpolate[interpolate_idx + if double { 2 } else { 1 }..];
 
                     let maybe_intersects = if lhs.len() > target_prefix.len() {
                         lhs.starts_with(target_prefix)
@@ -1794,6 +1832,7 @@ impl<'a> Runner<'a> {
                     return Ok(());
                 }
                 let mut is_interpolate = None;
+                let mut double_interpolate = false;
                 let display_name = job.display_name(&self.tasks);
 
                 let task_num = job.task;
@@ -1808,6 +1847,7 @@ impl<'a> Runner<'a> {
                             return Err(anyhow!("Error processing target '{}' in task {} - can only have a single interpolation target per task", &target, &display_name));
                         }
                         is_interpolate = Some(target.clone());
+                        double_interpolate = target.contains("##");
                     }
                     job_targets.push(target.to_string());
                 }
@@ -1824,6 +1864,7 @@ impl<'a> Runner<'a> {
                 job.state = JobState::Initialized;
 
                 let mut expanded_interpolate = false;
+                let mut dep_double_interpolate = false;
                 let task_id = job.task;
                 let deps = task.deps.clone();
                 for dep in deps {
@@ -1834,6 +1875,7 @@ impl<'a> Runner<'a> {
                         if expanded_interpolate {
                             return Err(anyhow!("Error processing dep '{}' in task {} - only one interpolated deps is allowed", &dep, &display_name));
                         }
+                        dep_double_interpolate = dep.contains("##");
                         self.expand_interpolate(watcher, String::from(dep), job_num, task_num)
                             .await?;
                         expanded_interpolate = true;
@@ -1869,12 +1911,24 @@ impl<'a> Runner<'a> {
                     }
                 }
 
-                if is_interpolate.is_some() && !expanded_interpolate {
-                    return Err(anyhow!(
-                        "Task {} defines an interpolation target {} without an interpolation dep",
-                        &display_name,
-                        is_interpolate.unwrap()
-                    ));
+                if is_interpolate.is_some() {
+                    if !expanded_interpolate {
+                        return Err(anyhow!(
+                            "Task {} defines an interpolation target {} without an interpolation dep",
+                            &display_name,
+                            is_interpolate.unwrap()
+                        ));
+                    }
+                    if dep_double_interpolate != double_interpolate {
+                        return Err(anyhow!(
+                            "Task {} defines a {} interpolate target {} but with a {} interpolation dep. Dependency interpolation must use a '{}' interpolate to match.",
+                            &display_name,
+                            if double_interpolate { "double" } else { "single" },
+                            is_interpolate.unwrap(),
+                            if double_interpolate { "single" } else { "double" },
+                            if double_interpolate { "##" } else { "#" }
+                        ));
+                    }
                 }
             }
             Node::File(ref mut file) => {
@@ -1891,18 +1945,18 @@ impl<'a> Runner<'a> {
         parent_job: usize,
         parent_task: usize,
     ) -> Result<()> {
-        let interpolate_idx = dep.find('#').unwrap();
-        if dep[interpolate_idx + 1..].find('#').is_some() {
-            return Err(anyhow!("Multiple interpolates in '{}' not supported", &dep));
-        }
+        let (interpolate_idx, double) = find_interpolate(&dep)?.unwrap();
         let mut glob_target = String::new();
         glob_target.push_str(&dep[0..interpolate_idx]);
-        if glob_target.ends_with('/') || glob_target.ends_with('\\') {
+        if double {
+            if !glob_target.ends_with('/') && !glob_target.ends_with('\\') {
+                return Err(anyhow!("Unable to apply deep globbing to interpolate {}. Deep globbing interpolates are only supported for full paths with '##' immediately following a separator position.", &dep));
+            }
             glob_target.push_str("(**/*)");
         } else {
             glob_target.push_str("(*)");
         }
-        glob_target.push_str(&dep[interpolate_idx + 1..]);
+        glob_target.push_str(&dep[interpolate_idx + if double { 2 } else { 1 }..]);
         for entry in
             glob(&glob_target).expect(&format!("Failed to read glob pattern {}", &glob_target))
         {
@@ -1910,7 +1964,7 @@ impl<'a> Runner<'a> {
                 Ok(entry) => {
                     let dep_path = String::from(entry.path().to_str().unwrap()).replace('\\', "/");
                     let interpolate = &dep_path
-                        [interpolate_idx..dep_path.len() - dep.len() + interpolate_idx + 1];
+                        [interpolate_idx..dep_path.len() - dep.len() + interpolate_idx + if double { 2 } else { 1 }];
                     self.expand_interpolate_match(
                         watcher,
                         &dep_path,
@@ -1968,7 +2022,8 @@ impl<'a> Runner<'a> {
         job.state = JobState::Initialized;
 
         for parent_target in targets {
-            job.targets.push(parent_target.replace('#', interpolate));
+            job.targets
+                .push(replace_interpolate(&parent_target, interpolate));
         }
         let parent = self.get_job_mut(parent_job).unwrap();
         parent.deps.push(job_num);
